/*
 * Copyright (c) 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.cometd.server.http;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.jetty.client.CompletableResponseListener;
import org.eclipse.jetty.client.ContentResponse;
import org.eclipse.jetty.client.Request;
import org.eclipse.jetty.http.HttpStatus;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;

public class ServerRedeployTest extends AbstractBayeuxClientServerTest {
    @ParameterizedTest
    @MethodSource("transports")
    public void testServerRedeploy(Transport transport) throws Exception {
        startServer(transport, null);

        Request handshake = newBayeuxRequest("" +
                                             "[{" +
                                             "\"channel\": \"/meta/handshake\"," +
                                             "\"version\": \"1.0\"," +
                                             "\"minimumVersion\": \"1.0\"," +
                                             "\"supportedConnectionTypes\": [\"long-polling\"]" +
                                             "}]");
        ContentResponse response = handshake.send();
        Assertions.assertEquals(200, response.getStatus());

        String clientId = extractClientId(response);

        Request connect = newBayeuxRequest("" +
                                           "[{" +
                                           "\"channel\": \"/meta/connect\"," +
                                           "\"clientId\": \"" + clientId + "\"," +
                                           "\"connectionType\": \"long-polling\"," +
                                           "\"advice\": { \"timeout\": 0 }" +
                                           "}]");
        response = connect.send();
        Assertions.assertEquals(200, response.getStatus());

        connect = newBayeuxRequest("" +
                                   "[{" +
                                   "\"channel\": \"/meta/connect\"," +
                                   "\"clientId\": \"" + clientId + "\"," +
                                   "\"connectionType\": \"long-polling\"" +
                                   "}]");
        CompletableFuture<ContentResponse> futureResponse = new CompletableResponseListener(connect).send();

        // Wait for the /meta/connect to be suspended.
        // We cannot rely on the HeartBeatListener because the "suspended" event
        // happens before ServerSessionImpl.setScheduler() and the context stop
        // may not find the scheduler to cancel that wold produce the 500 response.
        Thread.sleep(1000);

        // Stop the context; this is the first half of a
        // redeploy since we don't restart the context.
        context.stop();

        // Expect the /meta/connect to be back with an exception.
        response = futureResponse.get(timeout * 2, TimeUnit.MILLISECONDS);
        Assertions.assertEquals(HttpStatus.INTERNAL_SERVER_ERROR_500, response.getStatus());
    }
}
